

/////////////////////////////////////////////
//          SPA TESTER for SPA.C           //
//                                         //
//      Solar Position Algorithm (SPA)     //
//                   for                   //
//        Solar Radiation Application      //
//                                         //
//             August 12, 2004             //
//                                         //
//   Filename: SPA_TESTER.C                //
//                                         //
//   Afshin Michael Andreas                //
//   afshin_andreas@nrel.gov (303)384-6383 //
//                                         //
//   Measurement & Instrumentation Team    //
//   Solar Radiation Research Laboratory   //
//   National Renewable Energy Laboratory  //
//   1617 Cole Blvd, Golden, CO 80401      //
/////////////////////////////////////////////

/////////////////////////////////////////////
// This sample program shows how to use    //
//    the SPA.C code.                      //
/////////////////////////////////////////////


/* Modified by Val Gretchev March 2015 */




#include <stdio.h>
#include "spa.h"  //include the SPA header file
#include "Buffers.h"
#include "USB_Driver.h"



spa_data spa;  //declare the SPA structure



void SPA_Test(void)
{

        int result;
        int hrs, min, sec;
        char* BufNum;
        char Code;
        char FormatBuf[64];
        long double a;

        //enter required input values into SPA structure
        spa.year            = 2003;
        spa.month           = 10;
        spa.day             = 17;
        spa.hour            = 12;
        spa.minute          = 30;
        spa.second          = 30;
        spa.delta_ut1       = 0;
        spa.delta_t         = 67;
        spa.timezone        = -7.0;
        spa.longitude       = -105.1786;
        spa.latitude        = 39.742476;
        spa.elevation       = 1830.14;
        spa.pressure        = 820;
        spa.temperature     = 11;
        spa.slope           = 30;
        spa.azm_rotation    = -10;
        spa.atmos_refract   = 0.5667;
        spa.function        = SPA_ALL;



        //call the SPA calculate function and pass the SPA structure
        result = spa_calculate(&spa);

        if (result == 0)  //check for SPA errors
        {
                //display the results inside the SPA structure
                USB_PutQueue("\r\n");
                
                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.jd, FormatBuf);
                (void)sprintf(BufNum,"Julian Day:    ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum,"\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.l, FormatBuf);
                (void)sprintf(BufNum,"L:             ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.b, FormatBuf);
                (void)sprintf(BufNum,"B:             ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.r, FormatBuf);
                (void)sprintf(BufNum,"R:             ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," AU\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.h, FormatBuf);
                (void)sprintf(BufNum,"H:             ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.del_psi, FormatBuf);
                (void)sprintf(BufNum,"Delta Psi:     ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.del_epsilon, FormatBuf);
                (void)sprintf(BufNum,"Delta Epsilon: ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.epsilon, FormatBuf);
                (void)sprintf(BufNum,"Epsilon:       ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                USB_PutQueue("\r\n");

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.zenith, FormatBuf);
                (void)sprintf(BufNum,"Zenith:        ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.azimuth_astro, FormatBuf);
                (void)sprintf(BufNum,"Azimuth_astro: ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.azimuth, FormatBuf);
                (void)sprintf(BufNum,"Azimuth:       ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.incidence, FormatBuf);
                (void)sprintf(BufNum,"Incidence:     ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                USB_PutQueue("\r\n");

                hrs = (int)(spa.sunrise);
                a = 60.0L * (spa.sunrise - hrs);
                min = (int)(a);
                a = 60.0L * (a - min);
                sec = (int)(a);
                BufNum = GetBuffer();
                (void)sprintf(BufNum,"Sunrise:       %02d:%02d:%02d Local Time\r\n", hrs, min, sec);
                USB_PutQueue(BufNum);

                hrs = (int)(spa.suntransit);
                a = 60.0L * (spa.suntransit - hrs);
                min = (int)(a);
                a = 60.0L * (a - min);
                sec = (int)(a);
                BufNum = GetBuffer();
                (void)sprintf(BufNum,"Suntransit:    %02d:%02d:%02d Local Time\r\n", hrs, min, sec);
                USB_PutQueue(BufNum);

                hrs = (int)(spa.sunset);
                a = 60.0L * (spa.sunset - hrs);
                min = (int)(a);
                a = 60.0L * (a - min);
                sec = (int)(a);
                BufNum = GetBuffer();
                (void)sprintf(BufNum,"Sunset:        %02d:%02d:%02d Local Time\r\n", hrs, min, sec);
                USB_PutQueue(BufNum);

                USB_PutQueue("\r\n");
                
                BufNum = GetBuffer();
                Code = LongDoubleToStr(spa.azimuth - 180.0L, FormatBuf);
                (void)sprintf(BufNum,"Pan angle from South:       ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);

                BufNum = GetBuffer();
                Code = LongDoubleToStr(90.0L - spa.zenith, FormatBuf);
                (void)sprintf(BufNum,"Tilt angle from horizontal: ");
                strcat(BufNum,FormatBuf);
                strcat(BufNum," degrees\r\n");
                USB_PutQueue(BufNum);
        }
        else
        {
                BufNum = GetBuffer();
                (void)sprintf(BufNum,"SPA Error Code: %d\r\n", result);
                USB_PutQueue(BufNum);
        }
}





/////////////////////////////////////////////
// The output of this program should be:
//
//Julian Day:    2452930.312847
//L:             2.401826e+01 degrees
//B:             -1.011219e-04 degrees
//R:             0.996542 AU
//H:             11.105902 degrees
//Delta Psi:     -3.998404e-03 degrees
//Delta Epsilon: 1.666568e-03 degrees
//Epsilon:       23.440465 degrees
//Zenith:        50.111622 degrees
//Azimuth:       194.340241 degrees
//Incidence:     25.187000 degrees
//Sunrise:       06:12:43 Local Time
//Sunset:        17:20:19 Local Time
//
/////////////////////////////////////////////